#include "stdafx.h"
#include "Main.h"
#include "Replacer.h"

static char g_szClassName[] = "RefReplacerForDSPCB";  // Window class name.
static HINSTANCE g_hInstance = NULL;                  // Instance.
static HWND g_hWnd = NULL;                            // Dialog window handle.

static LRESULT CALLBACK DialogProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp);
static void SetWinCenter(HWND hWnd);

static std::string MakeUpper(std::string const &src)
{
  std::string result;
  result.reserve(src.size());
  for (std::string::const_iterator i = src.begin(); i != src.end(); ++i)
  {
    result.push_back(toupper(*i));
  }
  return result;
}

/**
 * Main function.
 */
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR lpCmdLine, int nCmdShow)
{
  g_hInstance = hInstance;

  // Register window class.
  WNDCLASS wc;
  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc = DialogProc;  // Procedure function name
  wc.cbClsExtra = 0;
  wc.cbWndExtra = DLGWINDOWEXTRA;
  wc.hInstance = hInstance;     // Instance
  wc.hIcon = LoadIcon(NULL, IDI_HAND);
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
  wc.lpszMenuName = NULL;       // Menu name
  wc.lpszClassName = (LPCSTR)g_szClassName;
  if (!RegisterClass(&wc)) return -1;

  // Create new dialog.
  // WM_INITDIALOG message is sent in CreateDialog().
  HWND hWnd = CreateDialog(hInstance, MAKEINTRESOURCE(IDD_MAIN_DLG), 0, (DLGPROC)DialogProc);
  if (NULL == hWnd) return -1;

  // Show the window.
  SetWinCenter(g_hWnd);
  ShowWindow(g_hWnd, nCmdShow);
  UpdateWindow(g_hWnd);

  // Message loop.
  MSG msg;
  while (GetMessage(&msg, NULL, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

/**
 * Window procedure.
 */
static LRESULT CALLBACK DialogProc(HWND hWnd, UINT msg, WPARAM wp, LPARAM lp)
{
  switch (msg)
  {
  case WM_INITDIALOG:
    {
      // Initialize common controls.
      InitCommonControls();

      // Setup window icon.
      HICON hIcon = LoadIcon(g_hInstance, MAKEINTRESOURCE(IDI_MAIN_DLG));
      //SendMessage(hWnd, WM_SETICON, ICON_BIG, (LPARAM)hIcon);
      SendMessage(hWnd, WM_SETICON, ICON_SMALL, (LPARAM)hIcon);

      // Setup drag and drop of file.
      DragAcceptFiles(hWnd, TRUE);

      // Show startup message.
      g_hWnd = hWnd;
      AppendText("Refernce Replace Utility for DesignSpark PCB programmed by Euphorie!\n");
      Printf("Build date: %s\n", __DATE__);
      AppendText("\n");
      AppendText("Please drag and drop PCB files here!\n");
      SetProgBar(0, 100, 0);

      break;
    }
  case WM_DESTROY:
    {
		  PostQuitMessage(0);
		  return 0;
    }
  case WM_CTLCOLORSTATIC:
    {
      // Change color of read-only edit control.
      HDC hDC = (HDC)wp;
      HWND hCtrl = (HWND)lp;
      if (hCtrl == GetDlgItem(g_hWnd, IDC_EDIT1))
      {
        SetBkMode(hDC, OPAQUE);
        SetTextColor(hDC, RGB(0, 0, 0));
        SetBkColor(hDC, RGB(255, 255, 255));
        return (LRESULT)GetStockObject(WHITE_BRUSH);
      }
      break;
    }
  case WM_DROPFILES:
    {
      // Activate this window.
      SetForegroundWindow(hWnd);

      // Process drag and drop files.
      HDROP hDrop = (HDROP)wp;
      UINT uFileNum = DragQueryFile(hDrop, 0xffffffff, NULL, 0);
      for (UINT i = 0; i < uFileNum; ++i)
      {
        char szFileName[1024];
        DragQueryFile(hDrop, i, szFileName, sizeof(szFileName));
        std::string inputFileName(szFileName);

        AppendText("\n");
        Printf("Start to analyze \"%s\" ...\n", inputFileName.c_str());

        std::string base = inputFileName.substr(0, inputFileName.size() - 4);
        std::string ext = inputFileName.substr(inputFileName.size() - 4);
        ext = MakeUpper(ext);
        if (ext != ".PCB")
        {
          AppendText("Input file extension is invalid!\n");
          continue;
        }

        std::string outputFileName = base + "_replaced.pcb";

        bool result = ReplaceRefName(inputFileName, outputFileName);
        if (result) AppendText("COMPLETED!\n");
        else AppendText("FAILED!\n");
      }
      break;
    }
  case WM_COMMAND:
    {
      break;
    }
  case WM_CLOSE:
    {
      AppendText("\n");
      AppendText("See you...\n");
      Sleep(500);
      DestroyWindow(hWnd);
      break;
    }
  }
  return DefWindowProc(hWnd, msg, wp, lp);
}

static void SetWinCenter(HWND hWnd)
{
  HWND hDeskWnd;
  RECT deskrc, rc;
    
  hDeskWnd = GetDesktopWindow();
  GetWindowRect(hDeskWnd, (LPRECT)&deskrc);
  GetWindowRect(hWnd, (LPRECT)&rc);
  int x = (deskrc.right - (rc.right-rc.left)) / 2;
  int y = (deskrc.bottom - (rc.bottom-rc.top)) / 2;
  SetWindowPos(hWnd, HWND_TOP, x, y, (rc.right-rc.left), (rc.bottom-rc.top), SWP_SHOWWINDOW); 
}

void AppendText(LPCSTR szText)
{
  HWND hEdit = GetDlgItem(g_hWnd, IDC_EDIT1);
  int ndx = GetWindowTextLength(hEdit);

   SetFocus(hEdit);
   SendMessage(hEdit, EM_SETSEL, (WPARAM)ndx, (LPARAM)ndx);
   SendMessage(hEdit, EM_REPLACESEL, 0, (LPARAM)szText);
}

void Printf(char const *format, ...)
{
  va_list ap;

  va_start(ap, format);
  char buf[1024];
  vsprintf_s(buf, format, ap);
  AppendText(buf);
  va_end(ap);
}

void SetProgBar(short rangeMin, short rangeMax, short current)
{
  HWND hProg = GetDlgItem(g_hWnd, IDC_PROGRESS1);
  SendMessage(hProg, PBM_SETRANGE, (WPARAM)0, MAKELPARAM(rangeMin, rangeMax));
  SendMessage(hProg, PBM_SETPOS, current, 0);
}
